﻿using AutoMapper;
using Tuitio.Application.Commands;
using Tuitio.Application.Services;
using Tuitio.PublishedLanguage.Dto;
using MediatR;
using Microsoft.Extensions.Logging;
using System.Threading;
using System.Threading.Tasks;

namespace Tuitio.Application.CommandHandlers
{
    public class AuthorizeTokenHandler : IRequestHandler<AuthorizeToken, TokenCore>
    {
        private readonly IUserService _userService;
        private readonly IMapper _mapper;
        private readonly ILogger<AuthorizeTokenHandler> _logger;

        public AuthorizeTokenHandler(IUserService userService, IMapper mapper, ILogger<AuthorizeTokenHandler> logger)
        {
            _userService = userService;
            _mapper = mapper;
            _logger = logger;
        }

        public Task<TokenCore> Handle(AuthorizeToken command, CancellationToken cancellationToken)
        {
            var tokenCore = _userService.Authorize(command.Token);
            if (tokenCore == null)
            {
                _logger.LogDebug($"Authorization failed for token '{command.Token}'.");
                return null;
            }

            _logger.LogDebug($"Authorization succeeded for token '{command.Token}'.");
            var tokenCoreResult = _mapper.Map<TokenCore>(tokenCore);

            return Task.FromResult(tokenCoreResult);
        }
    }
}
